@@ -1,4 +1,4 @@
-use crate::config::{Config, WriteTypes};
+use crate::config::{Config, Unsafe, WriteTypes};
 use crate::converter::{CSharpTypeConverter, Converter};
 use interoptopus::lang::c::{CType, CompositeType, Constant, Documentation, EnumType, Field, FnPointerType, Function, Meta, PrimitiveType, Variant, Visibility};
 use interoptopus::patterns::api_guard::library_hash;
@@ -39,6 +39,14 @@ pub trait CSharpWriter {
         indented!(w, r#"using System.Collections.Generic;"#)?;
         indented!(w, r#"using System.Runtime.InteropServices;"#)?;
 
+        if self.config().use_unsafe == Unsafe::UnsafePlatformMemCpy {
+            indented!(w, r#"#if UNITY_2018_1_OR_NEWER"#)?;
+            indented!(w, r#"using Unity.Collections.LowLevel.Unsafe;"#)?;
+            indented!(w, r#"#else"#)?;
+            indented!(w, r#"using System.Runtime.CompilerServices;"#)?;
+            indented!(w, r#"#endif"#)?;
+        }
+
         for namespace_id in self.library().namespaces() {
             let namespace = self
                 .config()
@@ -205,7 +213,7 @@ pub trait CSharpWriter {
 
         indented!(w, r#"public static {} {}({}) {{"#, rval, name, params.join(", "))?;
 
-        if self.config().use_unsafe {
+        if self.config().use_unsafe.any_unsafe() {
 
             // unsafe
             //     {
@@ -652,7 +660,7 @@ pub trait CSharpWriter {
         indented!(w, [_ _], r#"{{"#)?;
         indented!(w, [_ _ _], r#"if (i >= Count) throw new IndexOutOfRangeException();"#)?;
 
-        if self.config().use_unsafe {
+        if self.config().use_unsafe.any_unsafe() {
             indented!(w, [_ _ _], r#"unsafe"#)?;
             indented!(w, [_ _ _], r#"{{"#)?;
             indented!(w, [_ _ _ _], r#"var d = ({}*) data.ToPointer();"#, type_string)?;
@@ -673,9 +681,24 @@ pub trait CSharpWriter {
         indented!(w, [_ _], r#"get"#)?;
         indented!(w, [_ _], r#"{{"#)?;
         indented!(w, [_ _ _], r#"var rval = new {}[len];"#, type_string)?;
-        indented!(w, [_ _ _], r#"for (var i = 0; i < (int) len; i++) {{"#)?;
-        indented!(w, [_ _ _ _], r#"rval[i] = this[i];"#)?;
-        indented!(w, [_ _ _], r#"}}"#)?;
+
+        if self.config().use_unsafe == Unsafe::UnsafePlatformMemCpy {
+            indented!(w, [_ _ _], r#"unsafe"#)?;
+            indented!(w, [_ _ _], r#"{{"#)?;
+            indented!(w, [_ _ _ _ ], r#"fixed (void* dst = rval)"#)?;
+            indented!(w, [_ _ _ _ ], r#"{{"#)?;
+            indented!(w, [_ _ _ _ _], r#"#if UNITY_2018_1_OR_NEWER"#)?;
+            indented!(w, [_ _ _ _ _], r#"UnsafeUtility.MemCpy(dst, data.ToPointer(), (long) (len * (ulong) sizeof({})));"#, type_string)?;
+            indented!(w, [_ _ _ _ _], r#"#else"#)?;
+            indented!(w, [_ _ _ _ _], r#"Unsafe.CopyBlock(dst, data.ToPointer(), (uint)len);"#)?;
+            indented!(w, [_ _ _ _ _], r#"#endif"#)?;
+            indented!(w, [_ _ _ _ ], r#"}}"#)?;
+            indented!(w, [_ _ _], r#"}}"#)?;
+        } else {
+            indented!(w, [_ _ _], r#"for (var i = 0; i < (int) len; i++) {{"#)?;
+            indented!(w, [_ _ _ _], r#"rval[i] = this[i];"#)?;
+            indented!(w, [_ _ _], r#"}}"#)?;
+        }
         indented!(w, [_ _ _], r#"return rval;"#)?;
         indented!(w, [_ _], r#"}}"#)?;
         indented!(w, [_], r#"}}"#)?;
@@ -741,7 +764,7 @@ pub trait CSharpWriter {
         indented!(w, [_ _], r#"get"#)?;
         indented!(w, [_ _], r#"{{"#)?;
         indented!(w, [_ _ _], r#"if (i >= Count) throw new IndexOutOfRangeException();"#)?;
-        if self.config().use_unsafe {
+        if self.config().use_unsafe.any_unsafe() {
             indented!(w, [_ _ _], r#"unsafe"#)?;
             indented!(w, [_ _ _], r#"{{"#)?;
             indented!(w, [_ _ _ _], r#"var d = ({}*) data.ToPointer();"#, type_string)?;
@@ -756,7 +779,7 @@ pub trait CSharpWriter {
         indented!(w, [_ _], r#"set"#)?;
         indented!(w, [_ _], r#"{{"#)?;
         indented!(w, [_ _ _], r#"if (i >= Count) throw new IndexOutOfRangeException();"#)?;
-        if self.config().use_unsafe {
+        if self.config().use_unsafe.any_unsafe() {
             indented!(w, [_ _ _], r#"unsafe"#)?;
             indented!(w, [_ _ _], r#"{{"#)?;
             indented!(w, [_ _ _ _], r#"var d = ({}*) data.ToPointer();"#, type_string)?;
@@ -776,9 +799,25 @@ pub trait CSharpWriter {
         indented!(w, [_ _], r#"get"#)?;
         indented!(w, [_ _], r#"{{"#)?;
         indented!(w, [_ _ _], r#"var rval = new {}[len];"#, type_string)?;
-        indented!(w, [_ _ _], r#"for (var i = 0; i < (int) len; i++) {{"#)?;
-        indented!(w, [_ _ _ _], r#"rval[i] = this[i];"#)?;
-        indented!(w, [_ _ _], r#"}}"#)?;
+
+        if self.config().use_unsafe == Unsafe::UnsafePlatformMemCpy {
+            indented!(w, [_ _ _], r#"unsafe"#)?;
+            indented!(w, [_ _ _], r#"{{"#)?;
+            indented!(w, [_ _ _ _ ], r#"fixed (void* dst = rval)"#)?;
+            indented!(w, [_ _ _ _ ], r#"{{"#)?;
+            indented!(w, [_ _ _ _ _], r#"#if UNITY_2018_1_OR_NEWER"#)?;
+            indented!(w, [_ _ _ _ _], r#"UnsafeUtility.MemCpy(dst, data.ToPointer(), (long) (len * (ulong) sizeof({})));"#, type_string)?;
+            indented!(w, [_ _ _ _ _], r#"#else"#)?;
+            indented!(w, [_ _ _ _ _], r#"Unsafe.CopyBlock(dst, data.ToPointer(), (uint)len);"#)?;
+            indented!(w, [_ _ _ _ _], r#"#endif"#)?;
+            indented!(w, [_ _ _ _ ], r#"}}"#)?;
+            indented!(w, [_ _ _], r#"}}"#)?;
+        } else {
+            indented!(w, [_ _ _], r#"for (var i = 0; i < (int) len; i++) {{"#)?;
+            indented!(w, [_ _ _ _], r#"rval[i] = this[i];"#)?;
+            indented!(w, [_ _ _], r#"}}"#)?;
+        }
+
         indented!(w, [_ _ _], r#"return rval;"#)?;
         indented!(w, [_ _], r#"}}"#)?;
         indented!(w, [_], r#"}}"#)?;
@@ -809,8 +848,9 @@ pub trait CSharpWriter {
 
     fn write_pattern_service(&self, w: &mut IndentWriter, class: &Service) -> Result<(), Error> {
         self.debug(w, "write_pattern_service")?;
-        let mut all_functions = vec![class.constructor().clone(), class.destructor().clone()];
+        let mut all_functions = class.constructors().to_vec();
         all_functions.extend_from_slice(class.methods());
+        all_functions.push(class.destructor().clone());
 
         let context_type_name = class.the_type().rust_name();
         let common_prefix = longest_common_prefix(&all_functions);
@@ -821,16 +861,18 @@ pub trait CSharpWriter {
         w.indent();
         indented!(w, r#"private IntPtr _context;"#)?;
 
-        // Ctor
-        let args = self.pattern_class_args_without_first_to_string(class.constructor(), true);
-        self.write_documentation(w, class.constructor().meta().documentation())?;
-        indented!(w, r#"public {}({})"#, context_type_name, args)?;
-        indented!(w, r#"{{"#)?;
-        w.indent();
-        self.write_pattern_service_success_enum_aware_rval(w, class, class.constructor(), false)?;
-        w.unindent();
-        indented!(w, r#"}}"#)?;
-        w.newline()?;
+        for ctor in class.constructors() {
+            // Ctor
+            let args = self.pattern_class_args_without_first_to_string(ctor, true);
+            self.write_documentation(w, ctor.meta().documentation())?;
+            indented!(w, r#"public {}({})"#, context_type_name, args)?;
+            indented!(w, r#"{{"#)?;
+            w.indent();
+            self.write_pattern_service_success_enum_aware_rval(w, class, ctor, false)?;
+            w.unindent();
+            indented!(w, r#"}}"#)?;
+            w.newline()?;
+        }
 
         // Dtor
         indented!(w, r#"public void Dispose()"#)?;
