@@ -2,9 +2,9 @@ use interoptopus::testing::assert_file_matches_generated;
 use interoptopus::util::NamespaceMappings;
 use interoptopus::Error;
 use interoptopus::Interop;
-use interoptopus_backend_csharp::{run_dotnet_command_if_installed, WriteTypes};
+use interoptopus_backend_csharp::{run_dotnet_command_if_installed, Unsafe, WriteTypes};
 
-fn generate_bindings_multi(prefix: &str, use_unsafe: bool) -> Result<(), Error> {
+fn generate_bindings_multi(prefix: &str, use_unsafe: Unsafe) -> Result<(), Error> {
     use interoptopus_backend_csharp::{Config, Generator};
 
     let library = interoptopus_reference_project::ffi_inventory();
@@ -44,8 +44,8 @@ fn generate_bindings_multi(prefix: &str, use_unsafe: bool) -> Result<(), Error>
 #[test]
 #[cfg_attr(miri, ignore)]
 fn bindings_match_reference() -> Result<(), Error> {
-    generate_bindings_multi("tests/output_safe/Interop", false)?;
-    generate_bindings_multi("tests/output_unsafe/Interop", true)?;
+    generate_bindings_multi("tests/output_safe/Interop", Unsafe::None)?;
+    generate_bindings_multi("tests/output_unsafe/Interop", Unsafe::UnsafePlatformMemCpy)?;
 
     assert_file_matches_generated("tests/output_safe/Interop.cs");
     assert_file_matches_generated("tests/output_safe/Interop.common.cs");
@@ -59,8 +59,8 @@ fn bindings_match_reference() -> Result<(), Error> {
 #[test]
 #[cfg_attr(miri, ignore)]
 fn bindings_work() -> Result<(), Error> {
-    generate_bindings_multi("tests/output_safe/Interop", false)?;
-    generate_bindings_multi("tests/output_unsafe/Interop", true)?;
+    generate_bindings_multi("tests/output_safe/Interop", Unsafe::None)?;
+    generate_bindings_multi("tests/output_unsafe/Interop", Unsafe::UnsafePlatformMemCpy)?;
 
     run_dotnet_command_if_installed("tests/output_safe/", "test")?;
     run_dotnet_command_if_installed("tests/output_unsafe/", "test")?;
@@ -70,6 +70,6 @@ fn bindings_work() -> Result<(), Error> {
 #[test]
 #[cfg_attr(miri, ignore)]
 fn prepare_benchmarks() -> Result<(), Error> {
-    generate_bindings_multi("benches/Interop", false)?;
+    generate_bindings_multi("benches/Interop", Unsafe::UnsafePlatformMemCpy)?;
     Ok(())
 }
